"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_transformer_core_1 = require("@aws-amplify/graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const lambda = __importStar(require("@aws-cdk/aws-lambda"));
const path = __importStar(require("path"));
const appsync = __importStar(require("@aws-cdk/aws-appsync"));
const iam = __importStar(require("@aws-cdk/aws-iam"));
class CountTransformer extends graphql_transformer_core_1.TransformerPluginBase {
    constructor() {
        super("count", "directive @count on OBJECT");
        this.object = (definition, directive, ctx) => {
            // Keep track of everything annotated with @count
            this.models.push(definition);
        };
        this.transformSchema = (ctx) => {
            var _a;
            const fields = [];
            // For each model that has been annotated with @count
            for (const model of this.models) {
                if (!((_a = model.directives) === null || _a === void 0 ? void 0 : _a.find((dir) => dir.name.value === "model"))) {
                    throw new Error("Any type annotated with @count must also be annotated with @model, as it re-uses types from that directive.");
                }
                // The top level field inside Query
                const queryName = (0, graphql_transformer_common_1.toCamelCase)(["count", model.name.value]);
                // The name of the filter argument to count()
                const filterInputName = (0, graphql_transformer_common_1.toPascalCase)([
                    "Model",
                    model.name.value,
                    "FilterInput",
                ]);
                // Make the actual Query field
                // e.g. countHits(filter: ModelHitFilterInput): Int
                fields.push((0, graphql_transformer_common_1.makeField)(queryName, [(0, graphql_transformer_common_1.makeInputValueDefinition)("filter", (0, graphql_transformer_common_1.makeNamedType)(filterInputName))], (0, graphql_transformer_common_1.makeNamedType)("Int")));
            }
            ctx.output.addQueryFields(fields);
        };
        this.generateResolvers = (ctx) => {
            // Path on the local filesystem to the handler zip file
            const HANDLER_LOCAL_PATH = path.join(__dirname, "handler.zip");
            const stack = ctx.stackManager.createStack("countResolverStack");
            const funcId = "countResolverFunc";
            const HANDLER_S3_PATH = `functions/${funcId}.zip`;
            // Create the resolver function. This can be shared across all tables since it receives a different payload
            // for each table
            const funcRole = new iam.Role(stack, `${funcId}LambdaRole`, {
                assumedBy: new iam.ServicePrincipal("lambda.amazonaws.com"),
            });
            const func = ctx.api.host.addLambdaFunction("countResolver", HANDLER_S3_PATH, "index.handler", HANDLER_LOCAL_PATH, lambda.Runtime.NODEJS_14_X, undefined, // layers
            funcRole, // execution role,
            undefined, // env vars
            undefined, // lambda timeout
            stack);
            funcRole.addManagedPolicy(iam.ManagedPolicy.fromAwsManagedPolicyName("service-role/AWSLambdaBasicExecutionRole"));
            // Make this lambda into a data source
            const dataSource = ctx.api.host.addLambdaDataSource(`countResolverDataSource`, func, {}, stack);
            for (const model of this.models) {
                // Find the table we want to scan
                const tableDataSource = ctx.dataSources.get(model);
                const table = tableDataSource.ds
                    .dynamoDbConfig;
                // Allow the lambda to access this table
                funcRole.addToPolicy(new iam.PolicyStatement({
                    actions: ["dynamodb:Scan"],
                    effect: iam.Effect.ALLOW,
                    resources: [
                        `arn:aws:dynamodb:${table.awsRegion}:${stack.account}:table/${table.tableName}`,
                    ],
                }));
                // Connect the resolver to the API
                const resolver = new appsync.CfnResolver(stack, `${model.name.value}CountResolver`, {
                    apiId: ctx.api.apiId,
                    fieldName: (0, graphql_transformer_common_1.toCamelCase)(["count", model.name.value]),
                    typeName: "Query",
                    kind: "UNIT",
                    dataSourceName: dataSource === null || dataSource === void 0 ? void 0 : dataSource.ds.attrName,
                    requestMappingTemplate: `
$util.toJson({
  "version": "2018-05-29",
  "operation": "Invoke",
  "payload": {
      "context": $ctx,
      "dynamo": $util.parseJson($util.transform.toDynamoDBFilterExpression($ctx.arguments.filter)),
      "tableName": "${table.tableName}"
  }
})
                `,
                    responseMappingTemplate: `
#if( $ctx.error )
    $util.error($ctx.error.message, $ctx.error.type)
#else
    $util.toJson($ctx.result)
#end
`,
                });
                // resolver.overrideLogicalId(resourceId);
                ctx.api.addSchemaDependency(resolver);
            }
        };
        this.models = [];
    }
}
exports.default = CountTransformer;
